@@ -48,6 +48,32 @@
  **/
 #define SC_VOID_DIR "/var/lib/snapd/void"
 
+/**
+ * Get the path to the mounted core snap on the host distribution.
+ *
+ * The core snap may be named just "core" (preferred) or "ubuntu-core"
+ * (legacy).  The mount point dependes on build-time configuration and may
+ * differ from distribution to distribution.
+ **/
+static const char *sc_get_outer_core_mount_point()
+{
+	const char *core_path = SNAP_MOUNT_DIR "/core/current/";
+	const char *ubuntu_core_path = SNAP_MOUNT_DIR "/ubuntu-core/current/";
+	static const char *result = NULL;
+	if (result == NULL) {
+		if (access(core_path, F_OK) == 0) {
+			// Use the "core" snap if available.
+			result = core_path;
+		} else if (access(ubuntu_core_path, F_OK) == 0) {
+			// If not try to fall back to the "ubuntu-core" snap.
+			result = ubuntu_core_path;
+		} else {
+			die("cannot locate the core snap");
+		}
+	}
+	return result;
+}
+
 static void setup_private_mount(const char *security_tag)
 {
 	uid_t uid = getuid();
@@ -254,7 +280,7 @@ static void setup_snappy_os_mounts()
 		die("cannot create temporary directory for the root file system");
 	}
 	// Bind mount the OS snap into the rootfs directory.
-	const char *core_snap_dir = SNAP_MOUNT_DIR "/ubuntu-core/current";
+	const char *core_snap_dir = sc_get_outer_core_mount_point();
 	debug("bind mounting core snap: %s -> %s", core_snap_dir, rootfs_dir);
 	if (mount(core_snap_dir, rootfs_dir, NULL, MS_BIND, NULL) != 0) {
 		die("cannot bind mount core snap: %s to %s", core_snap_dir,
@@ -323,13 +349,6 @@ static void setup_snappy_os_mounts()
 	sc_bind_mount_hostfs(rootfs_dir);
 	sc_mount_nvidia_driver(rootfs_dir);
 	sc_pivot_to_new_rootfs(rootfs_dir);
-	// Reset path as we cannot rely on the path from the host OS to
-	// make sense. The classic distribution may use any PATH that makes
-	// sense but we cannot assume it makes sense for the core snap
-	// layout. Note that the /usr/local directories are explicitly
-	// left out as they are not part of the core snap.
-	debug("resetting PATH to values in sync with core snap");
-	setenv("PATH", "/usr/sbin:/usr/bin:/sbin:/bin:/usr/games", 1);
 }
 
 /**
@@ -463,13 +482,14 @@ void sc_populate_mount_ns(const char *security_tag)
 	if (vanilla_cwd == NULL) {
 		die("cannot get the current working directory");
 	}
-	// Make our "/" a rslave of the real "/". this means that mounts from the
+	// Make our "/" a rslave of the real "/". This means that mounts from the
 	// host "/" get propagated to our namespace (i.e. we see new media mounts).
 	if (mount("none", "/", NULL, MS_REC | MS_SLAVE, NULL) != 0) {
 		die("can not make make / rslave");
 	}
+	bool on_classic = is_running_on_classic_distribution();
 	// do the mounting if run on a non-native snappy system
-	if (is_running_on_classic_distribution()) {
+	if (on_classic) {
 		setup_snappy_os_mounts();
 	}
 	// set up private mounts
@@ -479,8 +499,9 @@ void sc_populate_mount_ns(const char *security_tag)
 	setup_private_pts();
 
 	// setup quirks for specific snaps
-	sc_setup_quirks();
-
+	if (on_classic) {
+		sc_setup_quirks();
+	}
 	// setup the security backend bind mounts
 	sc_setup_mount_profiles(security_tag);
 
